use poise::serenity_prelude::{self as serenity, GatewayIntents};
use dotenv;
mod command;

pub struct Data {} // User data, which is stored and accessible in all command invocations
type Context<'a> = poise::Context<'a, Data, anyhow::Error>;

const DISCORD_INTENTS: &'static str = "DISCORD_INTENTS";
const DISCORD_TOKEN: &'static str = "DISCORD_TOKEN";
const DISCORD_ID: &'static str = "DISCORD_ID";

#[tokio::main]
async fn main() {
    // ======================================== READ ENV ======================================== 
    dotenv::dotenv().ok();
    let id: String = std::env::var(DISCORD_ID).expect(&format!("ERROR: Missing {DISCORD_ID}"));
    let token: String = std::env::var(DISCORD_TOKEN).expect(&format!("ERROR: Missing {DISCORD_TOKEN}"));
    let perm_env: String = std::env::var(DISCORD_INTENTS)
        .unwrap_or_else(|msg| {
            println!("DEBUG: what is: {msg}");
            println!("ERROR: Missing {DISCORD_INTENTS}");
            "0".to_string()
        });
    // parse into u64, then into GatewayIntents
    let perm_u64 = perm_env.parse::<u64>().unwrap_or(0u64);
    let intents: GatewayIntents = GatewayIntents::from_bits(perm_u64).unwrap_or(GatewayIntents::empty());
    // Re-build into string, just incase it has changed
    let perm_string = intents.bits().to_string();
    // ======================================== ENV DONE ======================================== 
     
    // Generate sick text like this: 
    // http://www.patorjk.com/software/taag/#p=testall&f=Graffiti&t=hello%20world
    println!(r#"

     /$$$$$$$$                                /$$             /$$                     /$$$$$$$              /$$    
    |__  $$__/                               | $$            | $$                    | $$__  $$            | $$    
       | $$  /$$$$$$  /$$$$$$/$$$$   /$$$$$$ | $$  /$$$$$$  /$$$$$$    /$$$$$$       | $$  \ $$  /$$$$$$  /$$$$$$  
       | $$ /$$__  $$| $$_  $$_  $$ /$$__  $$| $$ |____  $$|_  $$_/   /$$__  $$      | $$$$$$$  /$$__  $$|_  $$_/  
       | $$| $$$$$$$$| $$ \ $$ \ $$| $$  \ $$| $$  /$$$$$$$  | $$    | $$$$$$$$      | $$__  $$| $$  \ $$  | $$    
       | $$| $$_____/| $$ | $$ | $$| $$  | $$| $$ /$$__  $$  | $$ /$$| $$_____/      | $$  \ $$| $$  | $$  | $$ /$$
       | $$|  $$$$$$$| $$ | $$ | $$| $$$$$$$/| $$|  $$$$$$$  |  $$$$/|  $$$$$$$      | $$$$$$$/|  $$$$$$/  |  $$$$/
       |__/ \_______/|__/ |__/ |__/| $$____/ |__/ \_______/   \___/   \_______/      |_______/  \______/    \___/  
                                   | $$                                                                            
                                   | $$                                                                            
                                   |__/                                                                            
                                                                                                                        
    Template Bot.
    Invite this bot with:

    https://discord.com/api/oauth2/authorize?client_id={id}&permissions={perm_string}&scope=bot
    
    "#);
    
    // Use bitlogic "OR" to combine different intents.
    let intents =
        intents |
        GatewayIntents::non_privileged();


    let framework = poise::Framework::builder()
        .options(poise::FrameworkOptions {
            commands: vec![
                command::example(),
            ],
            ..Default::default()
        })
        // un-comment songbird from Cargo.toml first.
        //.client_settings(songbird::register)
        .token(token)
        .intents(intents)
        .setup(|ctx, _ready, framework| {
            Box::pin(async move {
                poise::builtins::register_globally(ctx, &framework.options().commands).await?;

                Ok(Data {})
            })
        });
    
    framework.build().await.unwrap().start().await.unwrap();
}

